"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBoundingRect = exports.getClientRects = void 0;
var sort = function (rects) {
    return rects.sort(function (A, B) {
        var top = A.top - B.top;
        if (top === 0) {
            return A.left - B.left;
        }
        return top;
    });
};
var overlaps = function (A, B) { return A.left <= B.left && B.left <= A.left + A.width; };
var sameLine = function (A, B, yMargin) {
    if (yMargin === void 0) { yMargin = 5; }
    return Math.abs(A.top - B.top) < yMargin && Math.abs(A.height - B.height) < yMargin;
};
var inside = function (A, B) {
    return A.top > B.top && A.left > B.left && A.top + A.height < B.top + B.height && A.left + A.width < B.left + B.width;
};
var nextTo = function (A, B, xMargin) {
    if (xMargin === void 0) { xMargin = 10; }
    var Aright = A.left + A.width;
    var Bright = B.left + B.width;
    return A.left <= B.left && Aright <= Bright && B.left - Aright <= xMargin;
};
var extendWidth = function (A, B) {
    // extend width of A to cover B
    A.width = Math.max(B.width - A.left + B.left, A.width);
};
var optimizeClientRects = function (clientRects) {
    var rects = sort(clientRects);
    var toRemove = new Set();
    var firstPass = rects.filter(function (rect) {
        return rects.every(function (otherRect) {
            return !inside(rect, otherRect);
        });
    });
    var passCount = 0;
    while (passCount <= 2) {
        firstPass.forEach(function (A) {
            firstPass.forEach(function (B) {
                if (A === B || toRemove.has(A) || toRemove.has(B)) {
                    return;
                }
                if (!sameLine(A, B)) {
                    return;
                }
                if (overlaps(A, B)) {
                    extendWidth(A, B);
                    A.height = Math.max(A.height, B.height);
                    toRemove.add(B);
                }
                if (nextTo(A, B)) {
                    extendWidth(A, B);
                    toRemove.add(B);
                }
            });
        });
        passCount += 1;
    }
    return firstPass.filter(function (rect) { return !toRemove.has(rect); });
};
var getClientRects = function (range, containerEl, shouldOptimize) {
    if (shouldOptimize === void 0) { shouldOptimize = true; }
    var clientRects = Array.from(range.getClientRects());
    var offset = containerEl.getBoundingClientRect();
    var rects = clientRects.map(function (rect) {
        return {
            top: rect.top + containerEl.scrollTop - offset.top,
            left: rect.left + containerEl.scrollLeft - offset.left,
            width: rect.width,
            height: rect.height,
        };
    });
    return shouldOptimize ? optimizeClientRects(rects) : rects;
};
exports.getClientRects = getClientRects;
var getBoundingRect = function (clientRects) {
    var rects = Array.from(clientRects).map(function (rect) {
        var left = rect.left, top = rect.top, width = rect.width, height = rect.height;
        var X0 = left;
        var X1 = left + width;
        var Y0 = top;
        var Y1 = top + height;
        return { X0: X0, X1: X1, Y0: Y0, Y1: Y1 };
    });
    var optimal = rects.reduce(function (res, rect) {
        return {
            X0: Math.min(res.X0, rect.X0),
            X1: Math.max(res.X1, rect.X1),
            Y0: Math.min(res.Y0, rect.Y0),
            Y1: Math.max(res.Y1, rect.Y1),
        };
    }, rects[0]);
    var X0 = optimal.X0, X1 = optimal.X1, Y0 = optimal.Y0, Y1 = optimal.Y1;
    return {
        left: X0,
        top: Y0,
        width: X1 - X0,
        height: Y1 - Y0,
    };
};
exports.getBoundingRect = getBoundingRect;
